% TestEllipsesIntersection 
%
% Test suite for testing two ellipses intersection finding 
% routine.
%
% Usage:   TestEllipsesIntersection
%
% Author:
% Duan Omerevi
% Faculty of computer and information science
% University of Ljubljana
% http://vicos.fri.uni-lj.si/dusano/
%
% November 2005 - version 1.0
%
% Math sources:
% http://mathforum.org/dr.math/faq/faq.cubic.equations.html
% http://mathworld.wolfram.com/QuadraticEquation.html
% http://mathworld.wolfram.com/CircularSegment.html
% http://www.math.niu.edu/~rusin/known-math/95/ellipse.int
%
% Routine uses cubic equation solver by Herman Bruyninckx.
% See cubic.m for further details.

function TestEllipsesIntersection

    global test_count;
    test_count=0;
    
    TestEllipses([46.7529  648.8706    0.0075   -0.0015    0.0035],...
        [73.6483  656.0153    0.0031    0.0000    0.0007],42.713916);
    TestEllipses([98.6053  351.4474    0.0590    0.0008    0.0082],...
        [59.4286  398.0000    0.0428    0.0010    0.0023],-1);    
    TestEllipses([334.2412  221.9412    0.0047552   -5.7123e-006    0.0016207],...
        [333.9606  222.1823    0.004375   -1.2832e-005    0.0014638],1131.657698);    
    TestEllipses([102.98    399.69  0.0039526   0.00012862  0.0011723],...
        [102.95 399.63  0.0039139   0.00011096  0.0011659],1461.932981);
    TestEllipses([233.83    187.1   0.029717    0.031644    0.035302],...
        [233.7  187.66  0.016077    0.016359    0.018171],454.621423);
    TestEllipses([461.69    413.38  0.0012682   -2.0539e-005    0.00069651],...
        [463.49 415.62  0.0010819   -3.9127e-005    0.00047907],3343.459367);
    TestEllipses([840.96    394.21  0.0011226   0.0021079   0.0090848],...
        [850.4  336.29  0.10476 -0.0086486  0.00097976],47.238192);
    TestEllipses([869.8743280   513.1537810 0.0030019   -0.0011942  0.0006762],...
        [897.6450597    550.9709314 0.0183650   0.0124206   0.0092516],329.061550);
end

% Arguments:
% e1,e2 - parameters of ellipses
% assertValue - asserted value
function TestEllipses(e1,e2,assertValue)

    % Constant specifying when the asserted and real value are considered
    % equal
    ASSERT_EPS=1.0e-05;
    
    global test_count;
    test_count=test_count+1;
    
    [points,area]=EllipsesIntersection(e1,e2);
    
    M1=[e1(3) e1(4);e1(4) e1(5)];
    M2=[e2(3) e2(4);e2(4) e2(5)];
    
    figure;
    plot(e1(1),e1(2),'x');
    hold on;
    plot(e2(1),e2(2),'o');
    drawEllipse(M1,e1(1),e1(2),'r');    
    drawEllipse(M2,e2(1),e2(2),'b');
    axis equal;
    if area==-1
        title('No intersection');
    else
        title(sprintf('Area=%g',area));
    end
    
    if size(points,1)>0
        for i=1:size(points,1)
            plot(points(i,1),points(i,2),'o');
        end
    end
    
    if abs(area-assertValue)<ASSERT_EPS
        fprintf('Test %i has passed.\n',test_count);
    else
        fprintf('Test %i has failed. Asserted value=%f. Real value=%f.\n',test_count,assertValue,area);
    end
end

function drawEllipse(Mi,i,j,col)
    hold on;
    [v e]=eig(Mi);

    l1=1/sqrt(e(1));

    l2=1/sqrt(e(4));

    alpha=atan2(v(4),v(3));
    s=1;
    t = 0:pi/50:2*pi;
    y=s*(l2*sin(t));
    x=s*(l1*cos(t));

    xbar=x*cos(alpha) + y*sin(alpha);
    ybar=y*cos(alpha) - x*sin(alpha);
    plot(ybar+i,xbar+j,col,'LineWidth',1);
end