% TestUnitCircleEllipseIntersection 
%
% Test suite for testing an unit circle and an ellipse intersection finding 
% routine.
%
% Usage:   TestUnitCircleEllipseIntersection
%
% Author:
% Duan Omerevi
% Faculty of computer and information science
% University of Ljubljana
% http://vicos.fri.uni-lj.si/dusano/
%
% November 2005 - version 1.0
%
% Math sources:
% http://mathforum.org/dr.math/faq/faq.cubic.equations.html
% http://mathworld.wolfram.com/QuadraticEquation.html
% http://mathworld.wolfram.com/CircularSegment.html
% http://www.math.niu.edu/~rusin/known-math/95/ellipse.int
%
% Routine uses cubic equation solver by Herman Bruyninckx.
% See cubic.m for further details.

function TestUnitCircleEllipseIntersection
    global test_count;
    test_count=0;
    TestEllipse(1,1,2,1,1.201384);
    TestEllipse(1,0.5,1,0.5,0.600692);
    TestEllipse(1,0.5,2,0.5,1.321257);
    TestEllipse(1,0.25,2,0.75,2.123830);
    TestEllipse(0.75,0.25,2,0.5,1.633213);
    TestEllipse(1,1,1,2,1.201384);     % Checks if solution is a very small imaginery number
    TestEllipse(1,-1,2,1,1.201384);
    TestEllipse(1,0,2,1,2.750069);
    TestEllipse(2,0,2,1,1.032494);
    TestEllipse(0,2,1,2,1.032494);     % Checks if f==0
    TestEllipse(0,1.5,1,2,1.842555);
    TestEllipse(0,1,1,2,2.750069);
    TestEllipse(0,-0.9,1,0.1,0.156347);
    TestEllipse(0,0.75,1,2,3.001508);
    TestEllipse(0,0.5,1,2,3.102853);
    TestEllipse(2,2,2,1,-1);
    TestEllipse(0,0,2,1,pi);     % Checks if q==0
    TestEllipse(0,0.5,2,0.5,1.527507);
    TestEllipse(0,0,1,0.5,1.570796);
    TestEllipse(-1,0,2,0.5,1.527507);
    TestEllipse(0,0,2,1.5,pi);
    TestEllipse(0.25,0.25,0.60,0.5,0.942478);
    TestEllipse(0,0,2,1,pi);
    TestEllipse(1,1,1,1,0.570796);       % Checks intersection of two circles
    TestEllipse(0,0,1,1,pi);       % Checks intersection of two identical circles
    TestEllipse(-2,0,4,2,pi);
    TestEllipse(0,0,1,0.5,pi/2);
    TestEllipse(-0.0065688,-0.020646,1/sqrt(0.90275),1/sqrt(0.92048),pi);
    TestEllipse(-0.056059,0.017768,1/sqrt(0.8943),1/sqrt(1.0053),3.107029);
    TestEllipse(0.056153,-0.037511,1/sqrt(0.84791),1/sqrt(0.89043),pi);
    TestEllipse(-0.012487,0.018919,1/sqrt(1.0083),1/sqrt(0.99781),3.091257);
    TestEllipse(-0.011428,-0.014287,1/sqrt(1.029),1/sqrt(1.0439),3.030746);
    TestEllipse(0.3519730,-5.3045933,1/sqrt(171.8738207),1/sqrt(0.0281492),0.113396);
    TestEllipse(0.6800504,-0.5579751,1/sqrt(115.4958634),1/sqrt(0.2242371),0.255685);
end

% Arguments:
% x - x position of the second ellipse center
% y - y position of the second ellipse center
% a - major axis along x coordinate
% b - major axis along y coordinate
% assertValue - asserted value
function TestEllipse(x,y,a,b,assertValue)

    % Constant specifying when the asserted and real value are considered
    % equal
    ASSERT_EPS=1.0e-05;
    
    global test_count;
    test_count=test_count+1;
    
    M=[1/a^2 0;
        0 1/b^2];
    
    figure;   
    plot(0,0,'x');
    hold on;
    
    plot(x,y,'*');
    drawEllipse(eye(2),0,0,'r');    
    drawEllipse(M,x,y,'b');   
    
    axis([-3 3 -3 3]);
    axis equal;
    
    points=unitCircleEllipseIntersection(M(1),M(4),x,y);
    
    if size(points,1)>0
        for i=1:size(points,1)
            plot(points(i,1),points(i,2),'o');
        end
    end
    
    area=unitCircleEllipseAreaOfIntersection(M(1),M(4),x,y,points);
    
    title(sprintf('Area=%g',area));
    
    if abs(area-assertValue)<ASSERT_EPS
        fprintf('Test %i has passed.\n',test_count);
    else
        fprintf('Test %i has failed. Asserted value=%f. Real value=%f.\n',test_count,assertValue,area);
    end
end

function drawEllipse(Mi,i,j,col)
    hold on;
    [v e]=eig(Mi);

    l1=1/sqrt(e(1));

    l2=1/sqrt(e(4));

    alpha=atan2(v(4),v(3));
    s=1;
    t = 0:pi/50:2*pi;
    y=s*(l2*sin(t));
    x=s*(l1*cos(t));

    xbar=x*cos(alpha) + y*sin(alpha);
    ybar=y*cos(alpha) - x*sin(alpha);
    plot(ybar+i,xbar+j,col,'LineWidth',1);
end